#include <sys/types.h>
#include <sys/sysctl.h>
#include <sys/mman.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <libgen.h>
#include <libutil.h>

static char *progname;

static void
usage(int rc)
{
	fprintf(stderr, "Usage: %s -f /dev/vmm/vm2 -o 4G -l 2M 0x1fffff 0x2\n",
	    progname);
	exit(rc);
}

int
main(int argc, char *argv[])
{
	u_long offset;
	char *ptr, *fname;
	size_t mmap_len;
	uint64_t mmap_off;
	int fd, i, c;

	progname = basename(argv[0]);

	fname = NULL;
	mmap_off = 0;
	mmap_len = 4096;
	while ((c = getopt(argc, argv, "o:l:f:")) != -1) {
		switch (c) {
		case 'f':
			fname = optarg;
			break;
		case 'l':
			if (expand_number(optarg, &mmap_len))
				usage(1);
			break;
		case 'o':
			if (expand_number(optarg, &mmap_off))
				usage(1);
			break;
		default:
			usage(1);
		}
	}

	if (fname == NULL)
		usage(1);

	argc -= optind;
	argv += optind;

	fd = open(fname, O_RDWR, 0);
	if (fd < 0) {
		perror("open");
		exit(1);
	}

	ptr = mmap(NULL, mmap_len, PROT_READ | PROT_WRITE,
	    MAP_NOCORE | MAP_SHARED, fd, mmap_off);
	if (ptr == MAP_FAILED) {
		perror("mmap");
		exit(1);
	}
	close(fd);

	printf("%s(%#lx/%#lx) mapped at %p\n", fname, mmap_off, mmap_len, ptr);
	for (i = 0; i < argc; i++) {
		offset = strtoul(argv[i], NULL, 0);
		printf("ptr[%#lx] = 0x%02X\n", offset, ptr[offset]);
	}

	exit(0);
}
